@@ -49,12 +49,23 @@ def preprocess(sentence, remove_punct, remove_stop_words, stemm, lang='en'):
         
         
 class sentence_tranformer():
-    def __init__(self, source_names, target_names, model=None):
+    def __init__(self, source_names, target_names, model=None, return_match_idx=False):
         if not source_names:
             raise Exception('Inputs are empty')
+        
+        if not target_names:
+            raise Exception('Targets are empty') 
+               
+        if pd.isnull(source_names).any():
+            raise Exception('Inputs contain null values')
+        
+        if pd.isnull(target_names).any():
+            raise Exception('Targets contain null values')
+        
         self.source_names = source_names
         self.target_names = [target  for target in target_names if not (pd.isnull(target))] 
         self.model = model
+        self.return_match_idx = return_match_idx
         # if no model is provided use the default model
         if model is None:
             print('initializing the model...')
@@ -86,45 +97,71 @@ class sentence_tranformer():
         sorted = -np.sort(-cosine_results)
         targets = []
         max_cosines = []
+        match_idxs = []
+        # loop over top results to extract the index, target, and score for each match
         for x in sorted[0]:
             if len(targets) == self.topn:
                 break
             for i in (np.where(cosine_results == x)[1]):
+                match_idxs.append(i)
                 targets.append(self.target_names[i])
                 max_cosines.append(x)
                 if len(targets) == self.topn:
                     break
 
-        return targets, max_cosines
+        return targets, max_cosines, match_idxs
     
 
     def top_cosine_sim(self):
         results = np.array([self.max_cosine_sim(self.model.encode(input)) if not (pd.isnull(input)) else (None, None) for input in self.source_names], dtype=object)
         self.targets = results[:, 0]
         self.top_cosine = results[:, 1]
+        self.match_idxs = results[:, 2]
 
     def _make_matchdf(self):
         ''' Build dataframe for result return '''
+        if not self.return_match_idx:
+            match_list = []
+            for source, targets, top_scores in zip(self.source_names, self.targets, self.top_cosine):
+                row = []
+                row.append(source)
+                if targets is not None:
+                    # loop over results of multi matches
+                    for target, top_score in zip(targets, top_scores):
+                        row.append(target)
+                        row.append(top_score) 
+                match_list.append(tuple(row))
 
-        # to list of tuples
-        match_list = []
-        for source, target, top_score in zip(self.source_names,self.targets, self.top_cosine):
-            row = []
-            row.append(source)
-            if target is not None:
-                for x, y in zip(target, top_score):
-                    row.append(x)
-                    row.append(y) 
-            match_list.append(tuple(row))
+            # prepare columns names
+            colnames = ['source', 'prediction', 'score']
+            
+            for i in range(2, self.topn+1):
+                colnames.append(f'prediction_{i}')
+                colnames.append(f'score_{i}')
 
-        # prepare columns names
-        colnames = ['source', 'prediction', 'score']
-        
-        for i in range(2, self.topn+1):
-            colnames.append(f'prediction_{i}')
-            colnames.append(f'score_{i}')
-    
-        match_df = pd.DataFrame(match_list, columns=colnames)
+            match_df = pd.DataFrame(match_list, columns=colnames)
+        else:
+            match_list = []
+            for source, targets, top_scores, match_idxs in zip(self.source_names, self.targets, self.top_cosine, self.match_idxs):
+                row = []
+                row.append(source)
+                if targets is not None:
+                    # loop over results of multi matches
+                    for target, top_score, match_idx in zip(targets, top_scores, match_idxs):
+                        row.append(target)
+                        row.append(top_score) 
+                        row.append(match_idx)
+                match_list.append(tuple(row))
+
+            # prepare columns names
+            colnames = ['source', 'prediction', 'score', 'match_idx']
+            
+            for i in range(2, self.topn+1):
+                colnames.append(f'prediction_{i}')
+                colnames.append(f'score_{i}')
+                colnames.append(f'match_idx_{i}')
+
+            match_df = pd.DataFrame(match_list, columns=colnames)  
 
         return match_df
 
@@ -134,6 +171,16 @@ class word_mover_distance():
     def __init__(self, source_names, target_names, model, return_match_idx=False):
         if not source_names:
             raise Exception('Inputs are empty')
+        
+        if not target_names:
+            raise Exception('Targets are empty') 
+               
+        if pd.isnull(source_names).any():
+            raise Exception('Inputs contain null values')
+        
+        if pd.isnull(target_names).any():
+            raise Exception('Targets contain null values')
+        
         self.source_names = source_names
         self.target_names = target_names
         self.model = model
@@ -165,16 +212,15 @@ class word_mover_distance():
         wmd_results = np.array([self.model.wmdistance(input, target) for target in self.target_names])
         
         # get topn results
-        wmd_sorted = np.sort(wmd_results)
+        wmd_sorted = np.sort(np.unique(wmd_results))
         scores = []
         indexes = []
         for x in wmd_sorted:
             if len(indexes) == self.topn:
                 break
-            # convert distance to score
-            scores.append(1 - x)
-            for y in np.where(wmd_results == x):
-                indexes.append(y[0])
+            for y in np.where(wmd_results == x)[0]:
+                scores.append(float(1 - x)) # convert distance to score
+                indexes.append(y)
                 if len(indexes) == self.topn:
                     break    
         targets = [self.target_names[idx] for idx in indexes]
@@ -184,7 +230,6 @@ class word_mover_distance():
             indexes.append(None)
             targets.append(None)
             scores.append(None)
-
         return targets, scores, indexes
     
 
